import 'dart:async';

import 'package:comunic/helpers/events_helper.dart';
import 'package:flutter/material.dart';

/// Little State hack to avoid issues
///
/// @author Pierre HUBERT

abstract class SafeState<T extends StatefulWidget> extends State<T> {
  final _subscriptions = List<StreamSubscription>();

  @override
  void dispose() {
    // Close subscriptions
    _subscriptions.forEach((f) => f.cancel());

    super.dispose();
  }

  @override
  void setState(fn) {
    if (mounted) super.setState(fn);
  }

  /// Register to a new subscription
  @protected
  void listen<T>(void onEvent(T event)) {
    _subscriptions.add(EventsHelper.on<T>(onEvent));
  }

  /// Register to a new subscription
  ///
  /// Callback will we called inside of setState
  @protected
  void listenChangeState<T>(void onEvent(T event)) {
    _subscriptions.add(EventsHelper.on<T>((d) {
      setState(() => onEvent(d));
    }));
  }
}
